@@ -17,16 +17,20 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from safeeyes import utility
+from safeeyes.translations import translate as _
 
 
 def validate(plugin_config, plugin_settings):
     command = None
     if utility.IS_WAYLAND:
-        if utility.DESKTOP_ENVIRONMENT == "gnome":
+        if (
+            utility.DESKTOP_ENVIRONMENT == "gnome"
+            or utility.DESKTOP_ENVIRONMENT == "kde"
+        ):
             return None
         command = "wlrctl"
     else:
-        command = "xprop"
+        return None
     if not utility.command_exist(command):
         return _("Please install the command-line tool '%s'") % command
     else:
